@@ -12,7 +12,8 @@ References
        Pattern Analysis and Machine Intelligence,
        IEEE Transactions on, vol. 22, no. 8, pp. 888-905, August 2000.
 """
-from skimage import data, io, segmentation, color
+
+from skimage import data, segmentation, color
 from skimage.future import graph
 from matplotlib import pyplot as plt
 
@@ -26,8 +27,12 @@ g = graph.rag_mean_color(img, labels1, mode='similarity')
 labels2 = graph.cut_normalized(labels1, g)
 out2 = color.label2rgb(labels2, img, kind='avg')
 
-plt.figure()
-io.imshow(out1)
-plt.figure()
-io.imshow(out2)
-io.show()
+fig, ax = plt.subplots(nrows=2, sharex=True, sharey=True, figsize=(6, 8))
+
+ax[0].imshow(out1)
+ax[1].imshow(out2)
+
+for a in ax:
+    a.axis('off')
+
+plt.tight_layout()
