@@ -1,16 +1,151 @@
-import six
+from __future__ import division
 import math
-import warnings
 import numpy as np
-from scipy import ndimage, spatial
+from scipy import spatial
 
-from skimage._shared.utils import get_bound_method_class, safe_as_int
-from skimage.util import img_as_float
-from ._warps_cy import _warp_fast
+from .._shared.utils import get_bound_method_class, safe_as_int
+
+
+def _to_ndimage_mode(mode):
+    """Convert from `numpy.pad` mode name to the corresponding ndimage mode."""
+    mode_translation_dict = dict(edge='nearest', symmetric='reflect',
+                                 reflect='mirror')
+    if mode in mode_translation_dict:
+        mode = mode_translation_dict[mode]
+    return mode
+
+
+def _center_and_normalize_points(points):
+    """Center and normalize image points.
+
+    The points are transformed in a two-step procedure that is expressed
+    as a transformation matrix. The matrix of the resulting points is usually
+    better conditioned than the matrix of the original points.
+
+    Center the image points, such that the new coordinate system has its
+    origin at the centroid of the image points.
+
+    Normalize the image points, such that the mean distance from the points
+    to the origin of the coordinate system is sqrt(2).
+
+    Parameters
+    ----------
+    points : (N, 2) array
+        The coordinates of the image points.
+
+    Returns
+    -------
+    matrix : (3, 3) array
+        The transformation matrix to obtain the new points.
+    new_points : (N, 2) array
+        The transformed image points.
+
+    References
+    ----------
+    .. [1] Hartley, Richard I. "In defense of the eight-point algorithm."
+           Pattern Analysis and Machine Intelligence, IEEE Transactions on 19.6
+           (1997): 580-593.
+
+    """
+
+    centroid = np.mean(points, axis=0)
+
+    rms = math.sqrt(np.sum((points - centroid) ** 2) / points.shape[0])
+
+    norm_factor = math.sqrt(2) / rms
+
+    matrix = np.array([[norm_factor, 0, -norm_factor * centroid[0]],
+                       [0, norm_factor, -norm_factor * centroid[1]],
+                       [0, 0, 1]])
+
+    pointsh = np.row_stack([points.T, np.ones((points.shape[0]),)])
+
+    new_pointsh = np.dot(matrix, pointsh).T
+
+    new_points = new_pointsh[:, :2]
+    new_points[:, 0] /= new_pointsh[:, 2]
+    new_points[:, 1] /= new_pointsh[:, 2]
+
+    return matrix, new_points
+
+
+def _umeyama(src, dst, estimate_scale):
+    """Estimate N-D similarity transformation with or without scaling.
+
+    Parameters
+    ----------
+    src : (M, N) array
+        Source coordinates.
+    dst : (M, N) array
+        Destination coordinates.
+    estimate_scale : bool
+        Whether to estimate scaling factor.
+
+    Returns
+    -------
+    T : (N + 1, N + 1)
+        The homogeneous similarity transformation matrix. The matrix contains
+        NaN values only if the problem is not well-conditioned.
+
+    References
+    ----------
+    .. [1] "Least-squares estimation of transformation parameters between two
+            point patterns", Shinji Umeyama, PAMI 1991, DOI: 10.1109/34.88573
+
+    """
+
+    num = src.shape[0]
+    dim = src.shape[1]
+
+    # Compute mean of src and dst.
+    src_mean = src.mean(axis=0)
+    dst_mean = dst.mean(axis=0)
+
+    # Subtract mean from src and dst.
+    src_demean = src - src_mean
+    dst_demean = dst - dst_mean
+
+    # Eq. (38).
+    A = np.dot(dst_demean.T, src_demean) / num
+
+    # Eq. (39).
+    d = np.ones((dim,), dtype=np.double)
+    if np.linalg.det(A) < 0:
+        d[dim - 1] = -1
+
+    T = np.eye(dim + 1, dtype=np.double)
+
+    U, S, V = np.linalg.svd(A)
+
+    # Eq. (40) and (43).
+    rank = np.linalg.matrix_rank(A)
+    if rank == 0:
+        return np.nan * T
+    elif rank == dim - 1:
+        if np.linalg.det(U) * np.linalg.det(V) > 0:
+            T[:dim, :dim] = np.dot(U, V)
+        else:
+            s = d[dim - 1]
+            d[dim - 1] = -1
+            T[:dim, :dim] = np.dot(U, np.dot(np.diag(d), V))
+            d[dim - 1] = s
+    else:
+        T[:dim, :dim] = np.dot(U, np.dot(np.diag(d), V.T))
+
+    if estimate_scale:
+        # Eq. (41) and (42).
+        scale = 1.0 / src_demean.var(axis=0).sum() * np.dot(S, d)
+    else:
+        scale = 1.0
+
+    T[:dim, dim] = dst_mean - scale * np.dot(T[:dim, :dim], src_mean.T)
+    T[:dim, :dim] *= scale
+
+    return T
 
 
 class GeometricTransform(object):
-    """Perform geometric transformations on a set of coordinates.
+    """Base class for geometric transformations.
 
     """
     def __call__(self, coords):
@@ -24,7 +159,7 @@ class GeometricTransform(object):
         Returns
         -------
         coords : (N, 2) array
-            Transformed coordinates.
+            Destination coordinates.
 
         """
         raise NotImplementedError()
@@ -35,12 +170,12 @@ class GeometricTransform(object):
         Parameters
         ----------
         coords : (N, 2) array
-            Source coordinates.
+            Destination coordinates.
 
         Returns
         -------
         coords : (N, 2) array
-            Transformed coordinates.
+            Source coordinates.
 
         """
         raise NotImplementedError()
@@ -64,7 +199,6 @@ class GeometricTransform(object):
             Residual for coordinate.
 
         """
-
         return np.sqrt(np.sum((self(src) - dst)**2, axis=1))
 
     def __add__(self, other):
@@ -74,8 +208,292 @@ class GeometricTransform(object):
         raise NotImplementedError()
 
 
+class FundamentalMatrixTransform(GeometricTransform):
+    """Fundamental matrix transformation.
+
+    The fundamental matrix relates corresponding points between a pair of
+    uncalibrated images. The matrix transforms homogeneous image points in one
+    image to epipolar lines in the other image.
+
+    The fundamental matrix is only defined for a pair of moving images. In the
+    case of pure rotation or planar scenes, the homography describes the
+    geometric relation between two images (`ProjectiveTransform`). If the
+    intrinsic calibration of the images is known, the essential matrix describes
+    the metric relation between the two images (`EssentialMatrixTransform`).
+
+    References
+    ----------
+    .. [1] Hartley, Richard, and Andrew Zisserman. Multiple view geometry in
+           computer vision. Cambridge university press, 2003.
+
+    Parameters
+    ----------
+    matrix : (3, 3) array, optional
+        Fundamental matrix.
+
+    Attributes
+    ----------
+    params : (3, 3) array
+        Fundamental matrix.
+
+    """
+
+    def __init__(self, matrix=None):
+        if matrix is None:
+            # default to an identity transform
+            matrix = np.eye(3)
+        if matrix.shape != (3, 3):
+            raise ValueError("Invalid shape of transformation matrix")
+        self.params = matrix
+
+    def __call__(self, coords):
+        """Apply forward transformation.
+
+        Parameters
+        ----------
+        coords : (N, 2) array
+            Source coordinates.
+
+        Returns
+        -------
+        coords : (N, 3) array
+            Epipolar lines in the destination image.
+
+        """
+        coords_homogeneous = np.column_stack([coords, np.ones(coords.shape[0])])
+        return np.dot(coords_homogeneous, self.params.T)
+
+    def inverse(self, coords):
+        """Apply inverse transformation.
+
+        Parameters
+        ----------
+        coords : (N, 2) array
+            Destination coordinates.
+
+        Returns
+        -------
+        coords : (N, 3) array
+            Epipolar lines in the source image.
+
+        """
+        coords_homogeneous = np.column_stack([coords, np.ones(coords.shape[0])])
+        return np.dot(coords_homogeneous, self.params)
+
+    def _setup_constraint_matrix(self, src, dst):
+        """Setup and solve the homogeneous epipolar constraint matrix::
+
+            dst' * F * src = 0.
+
+        Parameters
+        ----------
+        src : (N, 2) array
+            Source coordinates.
+        dst : (N, 2) array
+            Destination coordinates.
+
+        Returns
+        -------
+        F_normalized : (3, 3) array
+            The normalized solution to the homogeneous system. If the system
+            is not well-conditioned, this matrix contains NaNs.
+        src_matrix : (3, 3) array
+            The transformation matrix to obtain the normalized source
+            coordinates.
+        dst_matrix : (3, 3) array
+            The transformation matrix to obtain the normalized destination
+            coordinates.
+
+        """
+        assert src.shape == dst.shape
+        assert src.shape[0] >= 8
+
+        # Center and normalize image points for better numerical stability.
+        try:
+            src_matrix, src = _center_and_normalize_points(src)
+            dst_matrix, dst = _center_and_normalize_points(dst)
+        except ZeroDivisionError:
+            self.params = np.full((3, 3), np.nan)
+            return 3 * [np.full((3, 3), np.nan)]
+
+        # Setup homogeneous linear equation as dst' * F * src = 0.
+        A = np.ones((src.shape[0], 9))
+        A[:, :2] = src
+        A[:, :3] *= dst[:, 0, np.newaxis]
+        A[:, 3:5] = src
+        A[:, 3:6] *= dst[:, 1, np.newaxis]
+        A[:, 6:8] = src
+
+        # Solve for the nullspace of the constraint matrix.
+        _, _, V = np.linalg.svd(A)
+        F_normalized = V[-1, :].reshape(3, 3)
+
+        return F_normalized, src_matrix, dst_matrix
+
+    def estimate(self, src, dst):
+        """Estimate fundamental matrix using 8-point algorithm.
+
+        The 8-point algorithm requires at least 8 corresponding point pairs for
+        a well-conditioned solution, otherwise the over-determined solution is
+        estimated.
+
+        Parameters
+        ----------
+        src : (N, 2) array
+            Source coordinates.
+        dst : (N, 2) array
+            Destination coordinates.
+
+        Returns
+        -------
+        success : bool
+            True, if model estimation succeeds.
+
+        """
+
+        F_normalized, src_matrix, dst_matrix = \
+            self._setup_constraint_matrix(src, dst)
+
+        # Enforcing the internal constraint that two singular values must be
+        # non-zero and one must be zero.
+        U, S, V = np.linalg.svd(F_normalized)
+        S[2] = 0
+        F = np.dot(U, np.dot(np.diag(S), V))
+
+        self.params = np.dot(dst_matrix.T, np.dot(F, src_matrix))
+
+        return True
+
+    def residuals(self, src, dst):
+        """Compute the Sampson distance.
+
+        The Sampson distance is the first approximation to the geometric error.
+
+        Parameters
+        ----------
+        src : (N, 2) array
+            Source coordinates.
+        dst : (N, 2) array
+            Destination coordinates.
+
+        Returns
+        -------
+        residuals : (N, ) array
+            Sampson distance.
+
+        """
+        src_homogeneous = np.column_stack([src, np.ones(src.shape[0])])
+        dst_homogeneous = np.column_stack([dst, np.ones(dst.shape[0])])
+
+        F_src = np.dot(self.params, src_homogeneous.T)
+        Ft_dst = np.dot(self.params.T, dst_homogeneous.T)
+
+        dst_F_src = np.sum(dst_homogeneous * F_src.T, axis=1)
+
+        return np.abs(dst_F_src) / np.sqrt(F_src[0] ** 2 + F_src[1] ** 2
+                                           + Ft_dst[0] ** 2 + Ft_dst[1] ** 2)
+
+
+class EssentialMatrixTransform(FundamentalMatrixTransform):
+    """Essential matrix transformation.
+
+    The essential matrix relates corresponding points between a pair of
+    calibrated images. The matrix transforms normalized, homogeneous image
+    points in one image to epipolar lines in the other image.
+
+    The essential matrix is only defined for a pair of moving images capturing a
+    non-planar scene. In the case of pure rotation or planar scenes, the
+    homography describes the geometric relation between two images
+    (`ProjectiveTransform`). If the intrinsic calibration of the images is
+    unknown, the fundamental matrix describes the projective relation between
+    the two images (`FundamentalMatrixTransform`).
+
+    References
+    ----------
+    .. [1] Hartley, Richard, and Andrew Zisserman. Multiple view geometry in
+           computer vision. Cambridge university press, 2003.
+
+    Parameters
+    ----------
+    rotation : (3, 3) array, optional
+        Rotation matrix of the relative camera motion.
+    translation : (3, 1) array, optional
+        Translation vector of the relative camera motion. The vector must
+        have unit length.
+    matrix : (3, 3) array, optional
+        Essential matrix.
+
+    Attributes
+    ----------
+    params : (3, 3) array
+        Essential matrix.
+
+    """
+
+    def __init__(self, rotation=None, translation=None, matrix=None):
+        if rotation is not None:
+            if translation is None:
+                raise ValueError("Both rotation and translation required")
+            if rotation.shape != (3, 3):
+                raise ValueError("Invalid shape of rotation matrix")
+            if abs(np.linalg.det(rotation) - 1) > 1e-6:
+                raise ValueError("Rotation matrix must have unit determinant")
+            if translation.size != 3:
+                raise ValueError("Invalid shape of translation vector")
+            if abs(np.linalg.norm(translation) - 1) > 1e-6:
+                raise ValueError("Translation vector must have unit length")
+            # Matrix representation of the cross product for t.
+            t_x = np.array([0, -translation[2], translation[1],
+                            translation[2], 0, -translation[0],
+                            -translation[1], translation[0], 0]).reshape(3, 3)
+            self.params = np.dot(t_x, rotation)
+        elif matrix is not None:
+            if matrix.shape != (3, 3):
+                raise ValueError("Invalid shape of transformation matrix")
+            self.params = matrix
+        else:
+            # default to an identity transform
+            self.params = np.eye(3)
+
+    def estimate(self, src, dst):
+        """Estimate essential matrix using 8-point algorithm.
+
+        The 8-point algorithm requires at least 8 corresponding point pairs for
+        a well-conditioned solution, otherwise the over-determined solution is
+        estimated.
+
+        Parameters
+        ----------
+        src : (N, 2) array
+            Source coordinates.
+        dst : (N, 2) array
+            Destination coordinates.
+
+        Returns
+        -------
+        success : bool
+            True, if model estimation succeeds.
+
+        """
+
+        E_normalized, src_matrix, dst_matrix = \
+            self._setup_constraint_matrix(src, dst)
+
+        # Enforcing the internal constraint that two singular values must be
+        # equal and one must be zero.
+        U, S, V = np.linalg.svd(E_normalized)
+        S[0] = (S[0] + S[1]) / 2.0
+        S[1] = S[0]
+        S[2] = 0
+        E = np.dot(U, np.dot(np.diag(S), V))
+
+        self.params = np.dot(dst_matrix.T, np.dot(E, src_matrix))
+
+        return True
+
+
 class ProjectiveTransform(GeometricTransform):
-    """Matrix transformation.
+    """Projective transformation.
 
     Apply a projective transformation (homography) on coordinates.
 
@@ -121,12 +539,6 @@ class ProjectiveTransform(GeometricTransform):
             raise ValueError("invalid shape of transformation matrix")
         self.params = matrix
 
-    @property
-    def _matrix(self):
-        warnings.warn('`_matrix` attribute is deprecated, '
-                      'use `params` instead.')
-        return self.params
-
     @property
     def _inv_matrix(self):
         return np.linalg.inv(self.params)
@@ -145,10 +557,7 @@ class ProjectiveTransform(GeometricTransform):
         return dst[:, :2]
 
     def __call__(self, coords):
-        return self._apply_mat(coords, self.params)
-
-    def inverse(self, coords):
-        """Apply inverse transformation.
+        """Apply forward transformation.
 
         Parameters
         ----------
@@ -158,14 +567,29 @@ class ProjectiveTransform(GeometricTransform):
         Returns
         -------
         coords : (N, 2) array
-            Transformed coordinates.
+            Destination coordinates.
+
+        """
+        return self._apply_mat(coords, self.params)
+
+    def inverse(self, coords):
+        """Apply inverse transformation.
+
+        Parameters
+        ----------
+        coords : (N, 2) array
+            Destination coordinates.
+
+        Returns
+        -------
+        coords : (N, 2) array
+            Source coordinates.
 
         """
         return self._apply_mat(coords, self._inv_matrix)
 
     def estimate(self, src, dst):
-        """Set the transformation matrix with the explicit transformation
-        parameters.
+        """Estimate the transformation from a set of corresponding points.
 
         You can determine the over-, well- and under-determined parameters
         with the total least-squares method.
@@ -214,7 +638,20 @@ class ProjectiveTransform(GeometricTransform):
         dst : (N, 2) array
             Destination coordinates.
 
+        Returns
+        -------
+        success : bool
+            True, if model estimation succeeds.
+
         """
+
+        try:
+            src_matrix, src = _center_and_normalize_points(src)
+            dst_matrix, dst = _center_and_normalize_points(dst)
+        except ZeroDivisionError:
+            self.params = np.nan * np.empty((3, 3))
+            return False
+
         xs = src[:, 0]
         ys = src[:, 1]
         xd = dst[:, 0]
@@ -247,8 +684,13 @@ class ProjectiveTransform(GeometricTransform):
         H.flat[list(self._coeffs) + [8]] = - V[-1, :-1] / V[-1, -1]
         H[2, 2] = 1
 
+        # De-center and de-normalize
+        H = np.dot(np.linalg.inv(dst_matrix), np.dot(H, src_matrix))
+
         self.params = H
 
+        return True
+
     def __add__(self, other):
         """Combine this transformation with another.
 
@@ -261,14 +703,17 @@ class ProjectiveTransform(GeometricTransform):
             else:
                 tform = ProjectiveTransform
             return tform(other.params.dot(self.params))
+        elif (hasattr(other, '__name__')
+                and other.__name__ == 'inverse'
+                and hasattr(get_bound_method_class(other), '_inv_matrix')):
+            return ProjectiveTransform(self._inv_matrix.dot(self.params))
         else:
             raise TypeError("Cannot combine transformations of differing "
                             "types.")
 
 
 class AffineTransform(ProjectiveTransform):
-
-    """2D affine transformation of the form::
+    """2D affine transformation of the form:
 
         X = a0*x + a1*y + a2 =
           = sx*x*cos(rotation) - sy*y*sin(rotation + shear) + a2
@@ -276,7 +721,7 @@ class AffineTransform(ProjectiveTransform):
         Y = b0*x + b1*y + b2 =
           = sx*x*sin(rotation) + sy*y*cos(rotation + shear) + b2
 
-    where ``sx`` and ``sy`` are zoom factors in the x and y directions,
+    where ``sx`` and ``sy`` are scale factors in the x and y directions,
     and the homogeneous transformation matrix is::
 
         [[a0  a1  a2]
@@ -359,7 +804,6 @@ class AffineTransform(ProjectiveTransform):
 
 
 class PiecewiseAffineTransform(GeometricTransform):
-
     """2D piecewise affine transformation.
 
     Control points are used to define the mapping. The transform is based on
@@ -382,7 +826,7 @@ class PiecewiseAffineTransform(GeometricTransform):
         self.inverse_affines = None
 
     def estimate(self, src, dst):
-        """Set the control points with which to perform the piecewise mapping.
+        """Estimate the transformation from a set of corresponding points.
 
         Number of source and destination coordinates must match.
 
@@ -393,6 +837,11 @@ class PiecewiseAffineTransform(GeometricTransform):
         dst : (N, 2) array
             Destination coordinates.
 
+        Returns
+        -------
+        success : bool
+            True, if model estimation succeeds.
+
         """
 
         # forward piecewise affine
@@ -415,6 +864,8 @@ class PiecewiseAffineTransform(GeometricTransform):
             affine.estimate(dst[tri, :], src[tri, :])
             self.inverse_affines.append(affine)
 
+        return True
+
     def __call__(self, coords):
         """Apply forward transformation.
 
@@ -486,21 +937,122 @@ class PiecewiseAffineTransform(GeometricTransform):
         return out
 
 
-class SimilarityTransform(ProjectiveTransform):
-    """2D similarity transformation of the form::
+class EuclideanTransform(ProjectiveTransform):
+    """2D Euclidean transformation of the form:
 
         X = a0 * x - b0 * y + a1 =
-          = m * x * cos(rotation) - m * y * sin(rotation) + a1
+          = x * cos(rotation) - y * sin(rotation) + a1
 
         Y = b0 * x + a0 * y + b1 =
-          = m * x * sin(rotation) + m * y * cos(rotation) + b1
+          = x * sin(rotation) + y * cos(rotation) + b1
 
-    where ``m`` is a zoom factor and the homogeneous transformation matrix is::
+    where the homogeneous transformation matrix is::
 
         [[a0  b0  a1]
          [b0  a0  b1]
          [0   0    1]]
 
+    The Euclidean transformation is a rigid transformation with rotation and
+    translation parameters. The similarity transformation extends the Euclidean
+    transformation with a single scaling factor.
+
+    Parameters
+    ----------
+    matrix : (3, 3) array, optional
+        Homogeneous transformation matrix.
+    rotation : float, optional
+        Rotation angle in counter-clockwise direction as radians.
+    translation : (tx, ty) as array, list or tuple, optional
+        x, y translation parameters.
+
+    Attributes
+    ----------
+    params : (3, 3) array
+        Homogeneous transformation matrix.
+
+    """
+
+    def __init__(self, matrix=None, rotation=None, translation=None):
+        params = any(param is not None
+                     for param in (rotation, translation))
+
+        if params and matrix is not None:
+            raise ValueError("You cannot specify the transformation matrix and"
+                             " the implicit parameters at the same time.")
+        elif matrix is not None:
+            if matrix.shape != (3, 3):
+                raise ValueError("Invalid shape of transformation matrix.")
+            self.params = matrix
+        elif params:
+            if rotation is None:
+                rotation = 0
+            if translation is None:
+                translation = (0, 0)
+
+            self.params = np.array([
+                [math.cos(rotation), - math.sin(rotation), 0],
+                [math.sin(rotation),   math.cos(rotation), 0],
+                [                 0,                    0, 1]
+            ])
+            self.params[0:2, 2] = translation
+        else:
+            # default to an identity transform
+            self.params = np.eye(3)
+
+    def estimate(self, src, dst):
+        """Estimate the transformation from a set of corresponding points.
+
+        You can determine the over-, well- and under-determined parameters
+        with the total least-squares method.
+
+        Number of source and destination coordinates must match.
+
+        Parameters
+        ----------
+        src : (N, 2) array
+            Source coordinates.
+        dst : (N, 2) array
+            Destination coordinates.
+
+        Returns
+        -------
+        success : bool
+            True, if model estimation succeeds.
+
+        """
+
+        self.params = _umeyama(src, dst, False)
+
+        return True
+
+    @property
+    def rotation(self):
+        return math.atan2(self.params[1, 0], self.params[1, 1])
+
+    @property
+    def translation(self):
+        return self.params[0:2, 2]
+
+
+class SimilarityTransform(EuclideanTransform):
+    """2D similarity transformation of the form:
+
+        X = a0 * x - b0 * y + a1 =
+          = s * x * cos(rotation) - s * y * sin(rotation) + a1
+
+        Y = b0 * x + a0 * y + b1 =
+          = s * x * sin(rotation) + s * y * cos(rotation) + b1
+
+    where ``s`` is a scale factor and the homogeneous transformation matrix is::
+
+        [[a0  b0  a1]
+         [b0  a0  b1]
+         [0   0    1]]
+
+    The similarity transformation extends the Euclidean transformation with a
+    single scaling factor in addition to the rotation and translation
+    parameters.
+
     Parameters
     ----------
     matrix : (3, 3) array, optional
@@ -551,38 +1103,13 @@ class SimilarityTransform(ProjectiveTransform):
             self.params = np.eye(3)
 
     def estimate(self, src, dst):
-        """Set the transformation matrix with the explicit parameters.
+        """Estimate the transformation from a set of corresponding points.
 
         You can determine the over-, well- and under-determined parameters
         with the total least-squares method.
 
         Number of source and destination coordinates must match.
 
-        The transformation is defined as::
-
-            X = a0 * x - b0 * y + a1
-            Y = b0 * x + a0 * y + b1
-
-        These equations can be transformed to the following form::
-
-            0 = a0 * x - b0 * y + a1 - X
-            0 = b0 * x + a0 * y + b1 - Y
-
-        which exist for each set of corresponding points, so we have a set of
-        N * 2 equations. The coefficients appear linearly so we can write
-        A x = 0, where::
-
-            A   = [[x 1 -y 0 -X]
-                   [y 0  x 1 -Y]
-                    ...
-                    ...
-                  ]
-            x.T = [a0 a1 b0 b1 c3]
-
-        In case of total least-squares the solution of this homogeneous system
-        of equations is the right singular vector of A which corresponds to the
-        smallest singular value normed by the coefficient c3.
-
         Parameters
         ----------
         src : (N, 2) array
@@ -590,33 +1117,16 @@ class SimilarityTransform(ProjectiveTransform):
         dst : (N, 2) array
             Destination coordinates.
 
+        Returns
+        -------
+        success : bool
+            True, if model estimation succeeds.
+
         """
-        xs = src[:, 0]
-        ys = src[:, 1]
-        xd = dst[:, 0]
-        yd = dst[:, 1]
-        rows = src.shape[0]
 
-        # params: a0, a1, b0, b1
-        A = np.zeros((rows * 2, 5))
-        A[:rows, 0] = xs
-        A[:rows, 2] = - ys
-        A[:rows, 1] = 1
-        A[rows:, 2] = xs
-        A[rows:, 0] = ys
-        A[rows:, 3] = 1
-        A[:rows, 4] = xd
-        A[rows:, 4] = yd
+        self.params = _umeyama(src, dst, True)
 
-        _, _, V = np.linalg.svd(A)
-
-        # solution is right singular vector that corresponds to smallest
-        # singular value
-        a0, a1, b0, b1 = - V[-1, :-1] / V[-1, -1]
-
-        self.params = np.array([[a0, -b0, a1],
-                                [b0,  a0, b1],
-                                [ 0,   0,  1]])
+        return True
 
     @property
     def scale(self):
@@ -627,17 +1137,9 @@ class SimilarityTransform(ProjectiveTransform):
             scale = self.params[0, 0] / math.cos(self.rotation)
         return scale
 
-    @property
-    def rotation(self):
-        return math.atan2(self.params[1, 0], self.params[1, 1])
-
-    @property
-    def translation(self):
-        return self.params[0:2, 2]
-
 
 class PolynomialTransform(GeometricTransform):
-    """2D transformation of the form::
+    """2D polynomial transformation of the form:
 
         X = sum[j=0:order]( sum[i=0:j]( a_ji * x**(j - i) * y**i ))
         Y = sum[j=0:order]( sum[i=0:j]( b_ji * x**(j - i) * y**i ))
@@ -664,15 +1166,8 @@ class PolynomialTransform(GeometricTransform):
             raise ValueError("invalid shape of transformation parameters")
         self.params = params
 
-    @property
-    def _params(self):
-        warnings.warn('`_params` attribute is deprecated, '
-                      'use `params` instead.')
-        return self.params
-
     def estimate(self, src, dst, order=2):
-        """Set the transformation matrix with the explicit transformation
-        parameters.
+        """Estimate the transformation from a set of corresponding points.
 
         You can determine the over-, well- and under-determined parameters
         with the total least-squares method.
@@ -714,6 +1209,11 @@ class PolynomialTransform(GeometricTransform):
         order : int, optional
             Polynomial order (number of coefficients is order + 1).
 
+        Returns
+        -------
+        success : bool
+            True, if model estimation succeeds.
+
         """
         xs = src[:, 0]
         ys = src[:, 1]
@@ -744,6 +1244,8 @@ class PolynomialTransform(GeometricTransform):
 
         self.params = params.reshape((2, u // 2))
 
+        return True
+
     def __call__(self, coords):
         """Apply forward transformation.
 
@@ -783,17 +1285,15 @@ class PolynomialTransform(GeometricTransform):
 
 
 TRANSFORMS = {
+    'euclidean': EuclideanTransform,
     'similarity': SimilarityTransform,
     'affine': AffineTransform,
     'piecewise-affine': PiecewiseAffineTransform,
     'projective': ProjectiveTransform,
+    'fundamental': FundamentalMatrixTransform,
+    'essential': EssentialMatrixTransform,
     'polynomial': PolynomialTransform,
 }
-HOMOGRAPHY_TRANSFORMS = (
-    SimilarityTransform,
-    AffineTransform,
-    ProjectiveTransform
-)
 
 
 def estimate_transform(ttype, src, dst, **kwargs):
@@ -806,13 +1306,14 @@ def estimate_transform(ttype, src, dst, **kwargs):
 
     Parameters
     ----------
-    ttype : {'similarity', 'affine', 'piecewise-affine', 'projective', \
-             'polynomial'}
+    ttype : {'euclidean', similarity', 'affine', 'piecewise-affine', \
+             'projective', 'polynomial'}
         Type of transform.
     kwargs : array or int
         Function parameters (src, dst, n, angle)::
 
             NAME / TTYPE        FUNCTION PARAMETERS
+            'euclidean'         `src, `dst`
             'similarity'        `src, `dst`
             'affine'            `src, `dst`
             'piecewise-affine'  `src, `dst`
@@ -886,265 +1387,3 @@ def matrix_transform(coords, matrix):
 
     """
     return ProjectiveTransform(matrix)(coords)
-
-
-def _stackcopy(a, b):
-    """Copy b into each color layer of a, such that::
-
-      a[:,:,0] = a[:,:,1] = ... = b
-
-    Parameters
-    ----------
-    a : (M, N) or (M, N, P) ndarray
-        Target array.
-    b : (M, N)
-        Source array.
-
-    Notes
-    -----
-    Color images are stored as an ``(M, N, 3)`` or ``(M, N, 4)`` arrays.
-
-    """
-    if a.ndim == 3:
-        a[:] = b[:, :, np.newaxis]
-    else:
-        a[:] = b
-
-
-def warp_coords(coord_map, shape, dtype=np.float64):
-    """Build the source coordinates for the output pixels of an image warp.
-
-    Parameters
-    ----------
-    coord_map : callable like GeometricTransform.inverse
-        Return input coordinates for given output coordinates.
-        Coordinates are in the shape (P, 2), where P is the number
-        of coordinates and each element is a ``(x, y)`` pair.
-    shape : tuple
-        Shape of output image ``(rows, cols[, bands])``.
-    dtype : np.dtype or string
-        dtype for return value (sane choices: float32 or float64).
-
-    Returns
-    -------
-    coords : (ndim, rows, cols[, bands]) array of dtype `dtype`
-            Coordinates for `scipy.ndimage.map_coordinates`, that will yield
-            an image of shape (orows, ocols, bands) by drawing from source
-            points according to the `coord_transform_fn`.
-
-    Notes
-    -----
-    This is a lower-level routine that produces the source coordinates used by
-    `warp()`.
-
-    It is provided separately from `warp` to give additional flexibility to
-    users who would like, for example, to re-use a particular coordinate
-    mapping, to use specific dtypes at various points along the the
-    image-warping process, or to implement different post-processing logic
-    than `warp` performs after the call to `ndimage.map_coordinates`.
-
-
-    Examples
-    --------
-    Produce a coordinate map that Shifts an image up and to the right:
-
-    >>> from skimage import data
-    >>> from scipy.ndimage import map_coordinates
-    >>>
-    >>> def shift_up10_left20(xy):
-    ...     return xy - np.array([-20, 10])[None, :]
-    >>>
-    >>> image = data.lena().astype(np.float32)
-    >>> coords = warp_coords(shift_up10_left20, image.shape)
-    >>> warped_image = map_coordinates(image, coords)
-
-    """
-    shape = safe_as_int(shape)
-    rows, cols = shape[0], shape[1]
-    coords_shape = [len(shape), rows, cols]
-    if len(shape) == 3:
-        coords_shape.append(shape[2])
-    coords = np.empty(coords_shape, dtype=dtype)
-
-    # Reshape grid coordinates into a (P, 2) array of (x, y) pairs
-    tf_coords = np.indices((cols, rows), dtype=dtype).reshape(2, -1).T
-
-    # Map each (x, y) pair to the source image according to
-    # the user-provided mapping
-    tf_coords = coord_map(tf_coords)
-
-    # Reshape back to a (2, M, N) coordinate grid
-    tf_coords = tf_coords.T.reshape((-1, cols, rows)).swapaxes(1, 2)
-
-    # Place the y-coordinate mapping
-    _stackcopy(coords[1, ...], tf_coords[0, ...])
-
-    # Place the x-coordinate mapping
-    _stackcopy(coords[0, ...], tf_coords[1, ...])
-
-    if len(shape) == 3:
-        coords[2, ...] = range(shape[2])
-
-    return coords
-
-
-def warp(image, inverse_map=None, map_args={}, output_shape=None, order=1,
-         mode='constant', cval=0., reverse_map=None):
-    """Warp an image according to a given coordinate transformation.
-
-    Parameters
-    ----------
-    image : 2-D or 3-D array
-        Input image.
-    inverse_map : transformation object, callable ``xy = f(xy, **kwargs)``, (3, 3) array
-        Inverse coordinate map. A function that transforms a (N, 2) array of
-        ``(x, y)`` coordinates in the *output image* into their corresponding
-        coordinates in the *source image* (e.g. a transformation object or its
-        inverse). See example section for usage.
-    map_args : dict, optional
-        Keyword arguments passed to `inverse_map`.
-    output_shape : tuple (rows, cols), optional
-        Shape of the output image generated. By default the shape of the input
-        image is preserved.  Note that, even for multi-band images, only rows
-        and columns need to be specified.
-    order : int, optional
-        The order of interpolation. The order has to be in the range 0-5:
-        * 0: Nearest-neighbor
-        * 1: Bi-linear (default)
-        * 2: Bi-quadratic
-        * 3: Bi-cubic
-        * 4: Bi-quartic
-        * 5: Bi-quintic
-    mode : string, optional
-        Points outside the boundaries of the input are filled according
-        to the given mode ('constant', 'nearest', 'reflect' or 'wrap').
-    cval : float, optional
-        Used in conjunction with mode 'constant', the value outside
-        the image boundaries.
-
-    Notes
-    -----
-    In case of a `SimilarityTransform`, `AffineTransform` and
-    `ProjectiveTransform` and `order` in [0, 3] this function uses the
-    underlying transformation matrix to warp the image with a much faster
-    routine.
-
-    Examples
-    --------
-    >>> from skimage.transform import warp
-    >>> from skimage import data
-    >>> image = data.camera()
-
-    The following image warps are all equal but differ substantially in
-    execution time. The image is shifted to the bottom.
-
-    Use a geometric transform to warp an image (fast):
-
-    >>> from skimage.transform import SimilarityTransform
-    >>> tform = SimilarityTransform(translation=(0, -10))
-    >>> warped = warp(image, tform)
-
-    Use a callable (slow):
-
-    >>> def shift_down(xy):
-    ...     xy[:, 1] -= 10
-    ...     return xy
-    >>> warped = warp(image, shift_down)
-
-    Use a transformation matrix to warp an image (fast):
-
-    >>> matrix = np.array([[1, 0, 0], [0, 1, -10], [0, 0, 1]])
-    >>> warped = warp(image, matrix)
-    >>> from skimage.transform import ProjectiveTransform
-    >>> warped = warp(image, ProjectiveTransform(matrix=matrix))
-
-    You can also use the inverse of a geometric transformation (fast):
-
-    >>> warped = warp(image, tform.inverse)
-
-    """
-    # Backward API compatibility
-    if reverse_map is not None:
-        warnings.warn('`reverse_map` parameter is deprecated and replaced by '
-                      'the `inverse_map` parameter.')
-        inverse_map = reverse_map
-
-    if image.ndim < 2 or image.ndim > 3:
-        raise ValueError("Input must have 2 or 3 dimensions.")
-
-    orig_ndim = image.ndim
-    image = np.atleast_3d(img_as_float(image))
-    ishape = np.array(image.shape)
-    bands = ishape[2]
-
-    if output_shape is None:
-        output_shape = ishape
-    else:
-        output_shape = safe_as_int(output_shape)
-
-
-    out = None
-
-    # use fast Cython version for specific interpolation orders and input
-    if order in range(4) and not map_args:
-
-        matrix = None
-
-        # inverse_map is a transformation matrix as numpy array
-        if isinstance(inverse_map, np.ndarray) and inverse_map.shape == (3, 3):
-            matrix = inverse_map
-
-        # inverse_map is a homography
-        elif isinstance(inverse_map, HOMOGRAPHY_TRANSFORMS):
-            matrix = inverse_map.params
-
-        # inverse_map is the inverse of a homography
-        elif (hasattr(inverse_map, '__name__')
-              and inverse_map.__name__ == 'inverse'
-              and get_bound_method_class(inverse_map) \
-                  in HOMOGRAPHY_TRANSFORMS):
-            matrix = np.linalg.inv(six.get_method_self(inverse_map).params)
-
-        if matrix is not None:
-            matrix = matrix.astype(np.double)
-            # transform all bands
-            dims = []
-            for dim in range(image.shape[2]):
-                dims.append(_warp_fast(image[..., dim], matrix,
-                                       output_shape=output_shape,
-                                       order=order, mode=mode, cval=cval))
-            out = np.dstack(dims)
-            if orig_ndim == 2:
-                out = out[..., 0]
-
-    if out is None:  # use ndimage.map_coordinates
-        rows, cols = output_shape[:2]
-
-        # inverse_map is a transformation matrix as numpy array
-        if isinstance(inverse_map, np.ndarray) and inverse_map.shape == (3, 3):
-            inverse_map = ProjectiveTransform(matrix=inverse_map)
-
-        def coord_map(*args):
-            return inverse_map(*args, **map_args)
-
-        coords = warp_coords(coord_map, (rows, cols, bands))
-
-        # Pre-filtering not necessary for order 0, 1 interpolation
-        prefilter = order > 1
-        out = ndimage.map_coordinates(image, coords, prefilter=prefilter,
-                                      mode=mode, order=order, cval=cval)
-
-    # The spline filters sometimes return results outside [0, 1],
-    # so clip to ensure valid data
-    clipped = np.clip(out, 0, 1)
-
-    if mode == 'constant' and not (0 <= cval <= 1):
-        clipped[out == cval] = cval
-
-    out = clipped
-
-    if out.ndim == 3 and orig_ndim == 2:
-        # remove singleton dimension introduced by atleast_3d
-        return out[..., 0]
-    else:
-        return out
